@@ -242,11 +242,21 @@ static int smp_fetch_var(const struct arg *args, struct sample *smp, const char
 
 	/* Check the availibity of the variable. */
 	switch (var_desc->scope) {
-	case SCOPE_SESS: vars = &smp->sess->vars;  break;
-	case SCOPE_TXN:  vars = &smp->strm->vars_txn;    break;
+	case SCOPE_SESS:
+		vars = &smp->sess->vars;
+		break;
+	case SCOPE_TXN:
+		if (!smp->strm)
+			return 0;
+		vars = &smp->strm->vars_txn;
+		break;
 	case SCOPE_REQ:
 	case SCOPE_RES:
-	default:         vars = &smp->strm->vars_reqres; break;
+	default:
+		if (!smp->strm)
+			return 0;
+		vars = &smp->strm->vars_reqres;
+		break;
 	}
 	if (vars->scope != var_desc->scope)
 		return 0;
@@ -369,7 +379,7 @@ static inline int sample_store_stream(const char *name, enum vars_scope scope, s
 /* Returns 0 if fails, else returns 1. */
 static int smp_conv_store(const struct arg *args, struct sample *smp, void *private)
 {
-	return sample_store_stream(args[0].data.var.name, args[1].data.var.scope, smp);
+	return sample_store_stream(args[0].data.var.name, args[0].data.var.scope, smp);
 }
 
 /* This fucntions check an argument entry and fill it with a variable
@@ -507,7 +517,7 @@ static enum act_return action_store(struct act_rule *rule, struct proxy *px,
 
 	/* Process the expression. */
 	memset(&smp, 0, sizeof(smp));
-	if (!sample_process(px, s->sess, s, dir|SMP_OPT_FINAL,
+	if (!sample_process(px, sess, s, dir|SMP_OPT_FINAL,
 	                    rule->arg.vars.expr, &smp))
 		return ACT_RET_CONT;
 
@@ -644,7 +654,7 @@ static int vars_max_size_reqres(char **args, int section_type, struct proxy *cur
 
 static struct sample_fetch_kw_list sample_fetch_keywords = {ILH, {
 
-	{ "var", smp_fetch_var, ARG1(1,STR), smp_check_var, SMP_T_STR, SMP_USE_HTTP_ANY },
+	{ "var", smp_fetch_var, ARG1(1,STR), smp_check_var, SMP_T_STR, SMP_USE_L5CLI },
 	{ /* END */ },
 }};
 
